@@ -1,5 +1,5 @@
 /* env - run a program in a modified environment
-   Copyright (C) 86, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1991-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -85,31 +85,57 @@
 
 #include "system.h"
 #include "error.h"
+#include "closeout.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "env"
+
+#define AUTHORS N_ ("Richard Mlynarik and David MacKenzie")
 
 int putenv ();
 
-static void usage __P ((int status));
-
 extern char **environ;
 
 /* The name by which this program was run. */
 char *program_name;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output and exit.  */
-static int show_version;
-
 static struct option const longopts[] =
 {
-  {"help", no_argument, &show_help, 1},
   {"ignore-environment", no_argument, NULL, 'i'},
   {"unset", required_argument, NULL, 'u'},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
+void
+usage (int status)
+{
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("\
+Usage: %s [OPTION]... [-] [NAME=VALUE]... [COMMAND [ARG]...]\n"),
+	      program_name);
+      fputs (_("\
+Set each NAME to VALUE in the environment and run COMMAND.\n\
+\n\
+  -i, --ignore-environment   start with an empty environment\n\
+  -u, --unset=NAME           remove variable from the environment\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
+\n\
+A mere - implies -i.  If no COMMAND, print the resulting environment.\n\
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+    }
+  exit (status);
+}
+
 int
 main (register int argc, register char **argv, char **envp)
 {
@@ -122,7 +148,9 @@ main (register int argc, register char **argv, char **envp)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  while ((optc = getopt_long (argc, argv, "+iu:", longopts, (int *) 0)) != EOF)
+  atexit (close_stdout);
+
+  while ((optc = getopt_long (argc, argv, "+iu:", longopts, NULL)) != -1)
     {
       switch (optc)
 	{
@@ -133,20 +161,13 @@ main (register int argc, register char **argv, char **envp)
 	  break;
 	case 'u':
 	  break;
+	case_GETOPT_HELP_CHAR;
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
 	default:
 	  usage (2);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("env (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (optind != argc && !strcmp (argv[optind], "-"))
     ignore_environment = 1;
 
@@ -158,7 +179,7 @@ main (register int argc, register char **argv, char **envp)
       putenv (*envp);
 
   optind = 0;			/* Force GNU getopt to re-initialize. */
-  while ((optc = getopt_long (argc, argv, "+iu:", longopts, (int *) 0)) != EOF)
+  while ((optc = getopt_long (argc, argv, "+iu:", longopts, NULL)) != -1)
     if (optc == 'u')
       putenv (optarg);		/* Requires GNU putenv. */
 
@@ -177,31 +198,10 @@ main (register int argc, register char **argv, char **envp)
     }
 
   execvp (argv[optind], &argv[optind]);
-  error (errno == ENOENT ? 127 : 126, errno, "%s", argv[optind]);
-}
 
-static void
-usage (int status)
-{
-  if (status != 0)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
-  else
-    {
-      printf (_("\
-Usage: %s [OPTION]... [-] [NAME=VALUE]... [COMMAND [ARG]...]\n"),
-	      program_name);
-      printf (_("\
-Set each NAME to VALUE in the environment and run COMMAND.\n\
-\n\
-  -i, --ignore-environment   start with an empty environment\n\
-  -u, --unset=NAME           remove variable from the environment\n\
-      --help                 display this help and exit\n\
-      --version              output version information and exit\n\
-\n\
-A mere - implies -i.  If no COMMAND, print the resulting environment.\n\
-"));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
-    }
-  exit (status);
+  {
+    int exit_status = (errno == ENOENT ? 127 : 126);
+    error (0, errno, "%s", argv[optind]);
+    exit (exit_status);
+  }
 }
