@@ -2,7 +2,7 @@
 
 /* Modified to run with the GNU shell by bfox. */
 
-/* Copyright (C) 1987-1999 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2002 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -122,18 +122,12 @@ static int term PARAMS ((void));
 static int and PARAMS ((void));
 static int or PARAMS ((void));
 
-#if __GNUC__ >= 2 && defined (__GNUC_MINOR__) \
-    && __GNUC_MINOR__ >= 5 && !defined (__STRICT_ANSI__)
-# define NO_RETURN_ATTRIBUTE __attribute__ ((noreturn))
-#else
-# define NO_RETURN_ATTRIBUTE /* empty */
-#endif
-
-static void test_syntax_error PARAMS ((char *format, char *arg)) NO_RETURN_ATTRIBUTE;
-static void beyond PARAMS ((void)) NO_RETURN_ATTRIBUTE;
+static void test_syntax_error PARAMS ((char const *format, char const *arg))
+     ATTRIBUTE_NORETURN;
+static void beyond PARAMS ((void)) ATTRIBUTE_NORETURN;
 
 static void
-test_syntax_error (char *format, char *arg)
+test_syntax_error (char const *format, char const *arg)
 {
   fprintf (stderr, "%s: ", argv[0]);
   fprintf (stderr, format, arg);
@@ -235,19 +229,19 @@ beyond (void)
 /* Syntax error for when an integer argument was expected, but
    something else was found. */
 static void
-integer_expected_error (char *pch)
+integer_expected_error (char const *pch)
 {
   test_syntax_error (_("integer expression expected %s\n"), pch);
 }
 
 /* Return nonzero if the characters pointed to by STRING constitute a
    valid number.  Stuff the converted number into RESULT if RESULT is
-   a non-null pointer to a long. */
+   not null.  */
 static int
-isint (register char *string, long int *result)
+isint (register char *string, intmax_t *result)
 {
   int sign;
-  long value;
+  intmax_t value;
 
   sign = 1;
   value = 0;
@@ -298,20 +292,16 @@ isint (register char *string, long int *result)
   return (1);
 }
 
-/* Find the modification time of FILE, and stuff it into AGE, a pointer
-   to a long.  Return nonzero if successful, else zero. */
+/* Find the modification time of FILE, and stuff it into *AGE.
+   Return 0 if successful, -1 if not.  */
 static int
-age_of (char *filename, long int *age)
+age_of (char *filename, time_t *age)
 {
   struct stat finfo;
-
-  if (test_stat (filename, &finfo) < 0)
-    return (0);
-
-  if (age)
+  int r = test_stat (filename, &finfo);
+  if (r == 0)
     *age = finfo.st_mtime;
-
-  return (1);
+  return r;
 }
 
 /*
@@ -394,7 +384,8 @@ binary_operator (void)
 {
   register int op;
   struct stat stat_buf, stat_spare;
-  long int l, r, value;
+  intmax_t l, r;
+  int value;
   /* Are the left and right integer expressions of the form '-l string'? */
   int l_is_l, r_is_l;
 
@@ -524,13 +515,14 @@ binary_operator (void)
 	  if (argv[op][2] == 't' && !argv[op][3])
 	    {
 	      /* nt - newer than */
+	      time_t lt, rt;
+	      int le, re;
 	      pos += 3;
 	      if (l_is_l || r_is_l)
 		test_syntax_error (_("-nt does not accept -l\n"), NULL);
-	      if (age_of (argv[op - 1], &l) && age_of (argv[op + 1], &r))
-		return (TRUE == (l > r));
-	      else
-		return (FALSE);
+	      le = age_of (argv[op - 1], &lt);
+	      re = age_of (argv[op + 1], &rt);
+	      return le > re || (le == 0 && lt > rt);
 	    }
 
 	  if (argv[op][2] == 'e' && !argv[op][3])
@@ -597,12 +589,14 @@ binary_operator (void)
 	  if ('t' == argv[op][2] && '\000' == argv[op][3])
 	    {
 	      /* ot - older than */
+	      time_t lt, rt;
+	      int le, re;
 	      pos += 3;
 	      if (l_is_l || r_is_l)
-		test_syntax_error (_("-nt does not accept -l\n"), NULL);
-	      if (age_of (argv[op - 1], &l) && age_of (argv[op + 1], &r))
-		return (TRUE == (l < r));
-	      return (FALSE);
+		test_syntax_error (_("-ot does not accept -l\n"), NULL);
+	      le = age_of (argv[op - 1], &lt);
+	      re = age_of (argv[op + 1], &rt);
+	      return le < re || (re == 0 && lt < rt);
 	    }
 	  break;
 	}
@@ -630,7 +624,7 @@ binary_operator (void)
 static int
 unary_operator (void)
 {
-  long fd, value;
+  int value;
   struct stat stat_buf;
 
   switch (argv[pos][1])
@@ -788,18 +782,21 @@ unary_operator (void)
 #endif
 
     case 't':			/* File (fd) is a terminal? */
-      advance (0);
-      if (pos < argc)
-	{
-	  if (!isint (argv[pos], &fd))
-	    integer_expected_error (_("after -t"));
-	  advance (0);
-	}
-      else
-	{
-	  fd = 1;
-	}
-      return (TRUE == (isatty ((int) fd)));
+      {
+	intmax_t fd;
+	advance (0);
+	if (pos < argc)
+	  {
+	    if (!isint (argv[pos], &fd))
+	      integer_expected_error (_("after -t"));
+	    advance (0);
+	  }
+	else
+	  {
+	    fd = 1;
+	  }
+	return (TRUE == (fd == (int) fd && isatty (fd)));
+      }
 
     case 'n':			/* True if arg has some length. */
       unary_advance ();
@@ -979,6 +976,7 @@ posixtest (void)
 
 #if defined (TEST_STANDALONE)
 # include "long-options.h"
+# include "closeout.h"
 
 void
 usage (int status)
@@ -994,25 +992,31 @@ Usage: %s EXPRESSION\n\
   or:  %s OPTION\n\
 "),
 	      program_name, program_name);
-      printf (_("\
+      fputs (_("\
 Exit with the status determined by EXPRESSION.\n\
 \n\
-  --help      display this help and exit\n\
-  --version   output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
 EXPRESSION is true or false and sets exit status.  It is one of:\n\
-"));
-      printf (_("\
+"), stdout);
+      fputs (_("\
 \n\
   ( EXPRESSION )               EXPRESSION is true\n\
   ! EXPRESSION                 EXPRESSION is false\n\
   EXPRESSION1 -a EXPRESSION2   both EXPRESSION1 and EXPRESSION2 are true\n\
   EXPRESSION1 -o EXPRESSION2   either EXPRESSION1 or EXPRESSION2 is true\n\
+"), stdout);
+      fputs (_("\
 \n\
   [-n] STRING          the length of STRING is nonzero\n\
   -z STRING            the length of STRING is zero\n\
   STRING1 = STRING2    the strings are equal\n\
   STRING1 != STRING2   the strings are not equal\n\
+"), stdout);
+      fputs (_("\
 \n\
   INTEGER1 -eq INTEGER2   INTEGER1 is equal to INTEGER2\n\
   INTEGER1 -ge INTEGER2   INTEGER1 is greater than or equal to INTEGER2\n\
@@ -1020,38 +1024,47 @@ EXPRESSION is true or false and sets exit status.  It is one of:\n\
   INTEGER1 -le INTEGER2   INTEGER1 is less than or equal to INTEGER2\n\
   INTEGER1 -lt INTEGER2   INTEGER1 is less than INTEGER2\n\
   INTEGER1 -ne INTEGER2   INTEGER1 is not equal to INTEGER2\n\
-"));
-      printf (_("\
+"), stdout);
+      fputs (_("\
 \n\
   FILE1 -ef FILE2   FILE1 and FILE2 have the same device and inode numbers\n\
   FILE1 -nt FILE2   FILE1 is newer (modification date) than FILE2\n\
   FILE1 -ot FILE2   FILE1 is older than FILE2\n\
+"), stdout);
+      fputs (_("\
 \n\
   -b FILE     FILE exists and is block special\n\
   -c FILE     FILE exists and is character special\n\
   -d FILE     FILE exists and is a directory\n\
   -e FILE     FILE exists\n\
+"), stdout);
+      fputs (_("\
   -f FILE     FILE exists and is a regular file\n\
   -g FILE     FILE exists and is set-group-ID\n\
+  -h FILE     FILE exists and is a symbolic link (same as -L)\n\
   -G FILE     FILE exists and is owned by the effective group ID\n\
   -k FILE     FILE exists and has its sticky bit set\n\
-  -L FILE     FILE exists and is a symbolic link\n\
+"), stdout);
+      fputs (_("\
+  -L FILE     FILE exists and is a symbolic link (same as -h)\n\
   -O FILE     FILE exists and is owned by the effective user ID\n\
   -p FILE     FILE exists and is a named pipe\n\
   -r FILE     FILE exists and is readable\n\
   -s FILE     FILE exists and has a size greater than zero\n\
+"), stdout);
+      fputs (_("\
   -S FILE     FILE exists and is a socket\n\
   -t [FD]     file descriptor FD (stdout by default) is opened on a terminal\n\
   -u FILE     FILE exists and its set-user-ID bit is set\n\
   -w FILE     FILE exists and is writable\n\
   -x FILE     FILE exists and is executable\n\
-"));
-      printf (_("\
+"), stdout);
+      fputs (_("\
 \n\
 Beware that parentheses need to be escaped (e.g., by backslashes) for shells.\n\
 INTEGER may also be -l STRING, which evaluates to the length of STRING.\n\
-"));
-      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -1061,7 +1074,7 @@ INTEGER may also be -l STRING, which evaluates to the length of STRING.\n\
 # define main test_command
 #endif
 
-#define AUTHORS "FIXME: ksb and mjb"
+#define AUTHORS N_ ("FIXME: ksb and mjb")
 
 /*
  * [:
@@ -1086,6 +1099,8 @@ main (int margc, char **margv)
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
+
+  atexit (close_stdout);
 #endif /* TEST_STANDALONE */
 
   argv = margv;
